package mandelbrot;

/**
 * Defines the basic algorithm for computing the Mandelbrot set (The abstract
 * class of the Template Method pattern).
 * 
 * @author ahoefer
 * 
 */
public abstract class AbstractMandelbrot {

	private static final double MAGNITUDE = 4.0;
	private static final int MAX_ITER = 1500;
	private static final double TOP_LEFT_REAL = -2;
	private static final double TOP_LEFT_IMAG = 1.5;
	private static final double BOTTOM_RIGHT_REAL = 1;
	private static final double BOTTOM_RIGHT_IMAG = -1.5;

	protected int width;
	protected int height;

	/**
	 * The created algorithm will compute a Mandelbrot set with the given width
	 * and height.
	 * 
	 * @param width
	 *            the width of the Mandelbrot set
	 * @param height
	 *            the height of the Mandelbrot set
	 */
	public AbstractMandelbrot(int width, int height) {
		this.width = width;
		this.height = height;
	}

	/**
	 * Triggers the computation of the Mandelbrot set. It also prints the
	 * computation's duration in milliseconds to the console.
	 * 
	 * @return the Mandelbrot
	 */
	public final int[][] computeMandelbrot() {
		long start = System.nanoTime();
		int[][] temp = compute();
		System.out.print((System.nanoTime() - start) / 1000000);
		return temp;
	}

	/**
	 * Computes the Mandelbrot set. This method has to be implemented by all
	 * sub-classes.
	 * 
	 * @return the Mandelbrot
	 */
	protected abstract int[][] compute();

	/**
	 * Computes one point of the Mandelbrot set.
	 * 
	 * @param cReal
	 *            the real part of the point
	 * @param cImag
	 *            the imaginary part of the point
	 * @return the number of iterations after which the computation stopped. If
	 *         this number exceeds the maximal number of iterations 0 is
	 *         returned.
	 */
	protected final int computeMandelbrotPoint(double cReal, double cImag) {
		double zReal = 0;
		double zImag = 0;
		double tmpReal;
		double tmpImag;
		int i = 0;
		do {
			i++;
			tmpReal = transformReal(zReal, zImag, cReal);
			tmpImag = transformImag(zReal, zImag, cImag);
			zReal = tmpReal;
			zImag = tmpImag;
		} while (((zReal * zReal + zImag * zImag) <= MAGNITUDE)
				&& (i <= MAX_ITER));
		return i % MAX_ITER;
	}

	/**
	 * Translates the x-value of a pixel in the Mandelbrot to the equivalent
	 * real part of a complex number c.
	 * 
	 * @param x
	 *            the x-value
	 * @return the real part
	 */
	protected final double translateReal(int x) {
		double real = TOP_LEFT_REAL + (double) x / height
				* (BOTTOM_RIGHT_REAL - TOP_LEFT_REAL);
		return real;
	}

	/**
	 * Translates the y-value of a pixel in the Mandelbrot to the equivalent
	 * imaginary part of a complex number c.
	 * 
	 * @param y
	 *            the y-value
	 * @return the imaginary part
	 */
	protected final double translateImag(int y) {
		double imag = TOP_LEFT_IMAG + (double) y / width
				* (BOTTOM_RIGHT_IMAG - TOP_LEFT_IMAG);
		return imag;
	}

	/**
	 * Computes one iteration of the Mandelbrot polynomial z(n+1) <- z(n)^2 + c
	 * (real part).
	 * 
	 * @param zReal
	 *            z(n)'s real part
	 * @param zImag
	 *            z(n)'s imaginary part
	 * @param cReal
	 *            c's real part
	 * @return z(n+1)'s real part
	 */
	protected final double transformReal(double zReal, double zImag,
			double cReal) {
		double real = (zReal * zReal - zImag * zImag) + cReal;
		return real;
	}

	/**
	 * Computes one iteration of the Mandelbrot polynomial z(n+1) <- z(n)^2 + c
	 * (imaginary part).
	 * 
	 * 
	 * @param zReal
	 *            z(n)'s real part
	 * @param zImag
	 *            z(n)'s imaginary part
	 * @param cImag
	 *            c's real part
	 * @return z(n+1)'s imaginary part
	 */
	protected final double transformImag(double zReal, double zImag,
			double cImag) {
		double imag = (2 * zReal * zImag) + cImag;
		return imag;
	}

	/**
	 * 
	 * @return the width of the Mandelbrot set
	 */
	public final int getWidth() {
		return width;
	}

	/**
	 * 
	 * @return the height of the Mandelbrot set
	 */
	public final int getHeight() {
		return height;
	}
}
