package mandelbrot;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;

import javax.swing.JPanel;

/**
 * A display for the Mandelbrot set.
 * 
 * @author ahoefer
 * 
 */
public class Display extends JPanel {

	private static final long serialVersionUID = -6482169096041908534L;

	private BufferedImage image;

	private AbstractMandelbrot mandelbrot;

	private ColorMap colorMap;

	/**
	 * Creates a new display.
	 * 
	 * @param mandelbrot
	 *            the Mandelbrot algorithm
	 * @param colorMap
	 *            the color map used to color the image of the Mandelbrot set
	 */
	public Display(AbstractMandelbrot mandelbrot, ColorMap colorMap) {
		this.mandelbrot = mandelbrot;
		this.colorMap = colorMap;
		setPreferredSize(new Dimension(mandelbrot.getWidth(), mandelbrot
				.getHeight()));
		updateImage();
	}

	/**
	 * Paints the image of the Mandelbrot set.
	 * @see {@link JPanel#paintComponent(Graphics)}
	 * @param g the graphics object
	 */
	@Override
	public void paintComponent(Graphics g) {
		g.drawImage(image, 0, 0, this);
	}

	private void updateImage() {
		int[][] mb = mandelbrot.computeMandelbrot();
		int h = mandelbrot.getHeight();
		int w = mandelbrot.getWidth();
		image = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
		WritableRaster raster = image.getRaster();
		for (int y = 0; y < h; y++) {
			for (int x = 0; x < w; x++) {
				raster.setPixel(x, y, getRGB(mb[x][y]));
			}
		}
	}

	private int[] getRGB(int mandelValue) {
		Color c = colorMap.getColor(mandelValue);
		int[] rgb = new int[3];
		rgb[0] = c.getRed();
		rgb[1] = c.getGreen();
		rgb[2] = c.getBlue();
		return rgb;
	}
}
